@@ -3,14 +3,13 @@ package RT::Interface::Email::Filter::TakeAction;
 use warnings;
 use strict;
 
-use RT::Interface::Email qw(ParseCcAddressesFromHead);
+use RT::Interface::Email;
 
-our @REGULAR_ATTRIBUTES = qw(Queue Owner Subject Status Priority FinalPriority);
+our @REGULAR_ATTRIBUTES = qw(Queue Subject Status Priority FinalPriority);
 our @TIME_ATTRIBUTES    = qw(TimeWorked TimeLeft TimeEstimated);
 our @DATE_ATTRIBUTES    = qw(Due Starts Started Resolved Told);
 our @LINK_ATTRIBUTES    = qw(MemberOf Parents Members Children
             HasMember RefersTo ReferredToBy DependsOn DependedOnBy);
-our @WATCHER_ATTRIBUTES = qw(Requestor Cc AdminCc);
 
 =head1 NAME
 
@@ -115,24 +114,17 @@ message.
 
 =head3 Custom field values
 
-Manage custom field values. Could be used multiple times.  (The curly braces
-are literal.)
+Manage custom field values. Could be used multiple times.
 
-    CustomField.{<CFName>}: <custom field value>
-    AddCustomField.{<CFName>}: <custom field value>
-    DelCustomField.{<CFName>}: <custom field value>
+    CustomField.{C<CFName>}: <custom field value>
+    AddCustomField.{C<CFName>}: <custom field value>
+    DelCustomField.{C<CFName>}: <custom field value>
 
 Short forms:
 
-    CF.{<CFName>}: <custom field value>
-    AddCF.{<CFName>}: <custom field value>
-    DelCF.{<CFName>}: <custom field value>
-
-=cut
-
-=head2 GetCurrentUser
-
-Returns a CurrentUser object.  Also performs all the commands.
+    CF.{C<CFName>}: <custom field value>
+    AddCF.{C<CFName>}: <custom field value>
+    DelCF.{C<CFName>}: <custom field value>
 
 =cut
 
@@ -152,8 +144,7 @@ sub GetCurrentUser {
         $RT::Logger->error(
             "Filter::TakeAction executed when "
             ."CurrentUser (actor) is not authorized. "
-            ."Most probably you want to add Auth::MailFrom plugin before "
-            ."Filter::TakeAction in the \@MailPlugins config."
+            ."Most probably you want to add Auth::MailFrom plugin before."
         );
         return ( $args{'CurrentUser'}, $args{'AuthLevel'} );
     }
@@ -164,53 +155,28 @@ sub GetCurrentUser {
         return ( $args{'CurrentUser'}, $args{'AuthLevel'} );
     }
 
-    # If only a particular group may perform commands by mail,
-    # bail out
-    my $new_config = RT->can('Config') && RT->Config->can('Get');
-    my $group_id = $new_config
-                 ? RT->Config->Get('CommandByMailGroup')
-                 : $RT::CommandByMailGroup;
-
-    if (defined $group_id) {
-        my $group = RT::Group->new($args{'CurrentUser'});
-        $group->Load($group_id);
-
-        if (!$group->HasMemberRecursively($args{'CurrentUser'}->PrincipalObj)) {
-            $RT::Logger->debug("CurrentUser not in CommandByMailGroup");
-            return ($args{'CurrentUser'}, $args{'AuthLevel'});
-        }
-    }
-
-    $RT::Logger->debug("Running CommandByMail as ".$args{'CurrentUser'}->UserObj->Name);
-
-    # find the content
     my @content;
     my @parts = $args{'Message'}->parts_DFS;
     foreach my $part (@parts) {
         my $body = $part->bodyhandle or next;
 
         #if it looks like it has pseudoheaders, that's our content
-        if ( $body->as_string =~ /^(?:\S+)(?:{.*})?:/m ) {
+        if ( $body->as_string =~ /^(?:\S+):/m ) {
             @content = $body->as_lines;
             last;
         }
     }
 
     my @items;
-    my $found_pseudoheaders = 0;
     foreach my $line (@content) {
-        next if $line =~ /^\s*$/ && ! $found_pseudoheaders;
-        last if $line !~ /^(?:(\S+(?:{.*})?)\s*?:\s*?(.*)\s*?|)$/;
-        $found_pseudoheaders = 1;
+        last if $line !~ /^(?:(\S+)\s*?:\s*?(.*)\s*?|)$/;
         push( @items, $1 => $2 );
-        $RT::Logger->debug("Found pseudoheader: $1 => $2");
     }
     my %cmds;
     while ( my $key = _CanonicalizeCommand( lc shift @items ) ) {
         my $val = shift @items;
         # strip leading and trailing spaces
         $val =~ s/^\s+|\s+$//g;
-        $RT::Logger->debug("Got command $key => $val");
 
         if ( exists $cmds{$key} ) {
             $cmds{$key} = [ $cmds{$key} ] unless ref $cmds{$key};
@@ -222,17 +188,6 @@ sub GetCurrentUser {
 
     my %results;
 
-    foreach my $cmd ( keys %cmds ) {
-        my ($val, $msg) = _CheckCommand( $cmd );
-        unless ( $val ) {
-            $results{ $cmd } = {
-                value   => delete $cmds{ $cmd },
-                result  => $val,
-                message => $msg,
-            };
-        }
-    }
-
     my $ticket_as_user = RT::Ticket->new( $args{'CurrentUser'} );
     my $queue          = RT::Queue->new( $args{'CurrentUser'} );
     if ( $cmds{'queue'} ) {
@@ -246,32 +201,18 @@ sub GetCurrentUser {
     # If we're updating.
     if ( $args{'Ticket'}->id ) {
         $ticket_as_user->Load( $args{'Ticket'}->id );
-        $RT::Logger->debug("Updating Ticket ".$ticket_as_user->Id." in Queue ".$queue->Name);
 
         # we set status later as correspond can reopen ticket
         foreach my $attribute (grep !/^(Status|TimeWorked)/, @REGULAR_ATTRIBUTES, @TIME_ATTRIBUTES) {
             next unless defined $cmds{ lc $attribute };
             next if $ticket_as_user->$attribute() eq $cmds{ lc $attribute };
 
-            # canonicalize owner -- accept an e-mail address
-            if ( $attribute eq 'Owner' && $cmds{ lc $attribute } =~ /\@/ ) {
-                my $user = RT::User->new($RT::SystemUser);
-                $user->LoadByEmail( $cmds{ lc $attribute } );
-                $cmds{ lc $attribute } = $user->Name if $user->id;
-            }
-
             _SetAttribute(
                 $ticket_as_user,        $attribute,
                 $cmds{ lc $attribute }, \%results
             );
         }
 
-        # we want the queue the ticket is currently in, not the queue
-        # that was passed to rt-mailgate, otherwise we can't find the
-        # proper set of Custom Fields.  But, we have to do this after 
-        # we potentially update the Queue from @REGULAR_ATTRIBUTES
-        $queue = $ticket_as_user->QueueObj();
-
         foreach my $attribute (@DATE_ATTRIBUTES) {
             next unless ( $cmds{ lc $attribute } );
 
@@ -285,7 +226,7 @@ sub GetCurrentUser {
             $results{ lc $attribute }->{value} = $cmds{ lc $attribute };
         }
 
-        foreach my $type ( @WATCHER_ATTRIBUTES ) {
+        foreach my $type ( qw(Requestor Cc AdminCc) ) {
             my %tmp = _ParseAdditiveCommand( \%cmds, 1, $type );
             next unless keys %tmp;
 
@@ -295,30 +236,24 @@ sub GetCurrentUser {
                 $args{'Ticket'}->$method->MemberEmailAddresses;
             } ];
             my ($add, $del) = _CompileAdditiveForUpdate( %tmp );
-            foreach my $text ( @$del ) {
-                my $user = RT::User->new($RT::SystemUser);
-                $user->LoadByEmail($text) if $text =~ /\@/;
-                $user->Load($text) unless $user->id;
+            foreach ( @$del ) {
                 my ( $val, $msg ) = $ticket_as_user->DeleteWatcher(
                     Type  => $type,
-                    PrincipalId => $user->PrincipalId,
+                    Email => $_,
                 );
                 push @{ $results{ 'Del'. $type } }, {
-                    value   => $text,
+                    value   => $_,
                     result  => $val,
                     message => $msg
                 };
             }
-            foreach my $text ( @$add ) {
-                my $user = RT::User->new($RT::SystemUser);
-                $user->LoadByEmail($text) if $text =~ /\@/;
-                $user->Load($text) unless $user->id;
+            foreach ( @$add ) {
                 my ( $val, $msg ) = $ticket_as_user->AddWatcher(
                     Type  => $type,
-                    PrincipalId => $user->PrincipalId,
+                    Email => $_,
                 );
                 push @{ $results{ 'Add'. $type } }, {
-                    value   => $text,
+                    value   => $_,
                     result  => $val,
                     message => $msg
                 };
@@ -327,15 +262,8 @@ sub GetCurrentUser {
 
         {
             my $time_taken = 0;
-            if (grep $_ eq 'TimeWorked', @TIME_ATTRIBUTES) {
-                if (ref $cmds{'timeworked'}) { 
-                    map { $time_taken += ($_ || 0) }  @{ $cmds{'timeworked'} };
-                    $RT::Logger->debug("Time taken: $time_taken");
-                }
-                else {
-                    $time_taken = $cmds{'timeworked'} || 0;
-                }
-            }
+            $time_taken = $cmds{'timeworked'} || 0
+                if grep $_ eq 'TimeWorked', @TIME_ATTRIBUTES;
 
             my $method = ucfirst $args{'Action'};
             my ($status, $msg) = $ticket_as_user->$method(
@@ -438,7 +366,7 @@ sub GetCurrentUser {
 
             _SetAttribute(
                 $ticket_as_user,        $attribute,
-                lc $cmds{ lc $attribute }, \%results
+                $cmds{ lc $attribute }, \%results
             );
         }
 
@@ -452,37 +380,18 @@ sub GetCurrentUser {
     } else {
 
         my %create_args = ();
-        foreach my $attribute (@REGULAR_ATTRIBUTES, @TIME_ATTRIBUTES) {
-            next unless exists $cmds{ lc $attribute };
-
-            # canonicalize owner -- accept an e-mail address
-            if ( $attribute eq 'Owner' && $cmds{ lc $attribute } =~ /\@/ ) {
-                my $user = RT::User->new($RT::SystemUser);
-                $user->LoadByEmail( $cmds{ lc $attribute } );
-                $cmds{ lc $attribute } = $user->Name if $user->id;
-            }
-
-            if ( $attribute eq 'TimeWorked' && ref $cmds{ lc $attribute } ) {
-                my $time_taken = 0;
-                map { $time_taken += ($_ || 0) }  @{ $cmds{'timeworked'} };
-                $cmds{'timeworked'} = $time_taken;
-                $RT::Logger->debug("Time taken on create: $time_taken");
-            }
-
-            if ( $attribute eq 'Status' && $cmds{ lc $attribute } ) {
-                $cmds{ lc $attribute } = lc $cmds{ lc $attribute };
-            }
-
-            $create_args{$attribute} = $cmds{ lc $attribute };
+        foreach my $attr (@REGULAR_ATTRIBUTES, @TIME_ATTRIBUTES) {
+            next unless exists $cmds{ lc $attr };
+            $create_args{$attr} = $cmds{ lc $attr };
         }
-        foreach my $attribute (@DATE_ATTRIBUTES) {
-            next unless exists $cmds{ lc $attribute };
+        foreach my $attr (@DATE_ATTRIBUTES) {
+            next unless exists $cmds{ lc $attr };
             my $date = RT::Date->new( $args{'CurrentUser'} );
             $date->Set(
                 Format => 'unknown',
-                Value  => $cmds{ lc $attribute }
+                Value  => $cmds{ lc $attr }
             );
-            $create_args{$attribute} = $date->ISO;
+            $create_args{$attr} = $date->ISO;
         }
 
         # Canonicalize links
@@ -502,7 +411,7 @@ sub GetCurrentUser {
 
         # Canonicalize watchers
         # First of all fetch default values
-        foreach my $type ( @WATCHER_ATTRIBUTES ) {
+        foreach my $type ( qw(Requestor Cc AdminCc) ) {
             my %tmp = _ParseAdditiveCommand( \%cmds, 1, $type );
             $tmp{'Default'} = [ $args{'CurrentUser'}->EmailAddress ] if $type eq 'Requestor';
             $tmp{'Default'} = [
@@ -546,8 +455,6 @@ sub GetCurrentUser {
             return ($args{'CurrentUser'}, $args{'AuthLevel'});
         }
 
-        _ReportResults( Results => \%results, Message => $args{'Message'} );
-
         # now that we've created a ticket, we abort so we don't create another.
         $args{'Ticket'}->Load( $id );
         return ( $args{'CurrentUser'}, -2 );
@@ -644,6 +551,7 @@ sub _SetAttribute {
         result  => $val,
         message => $msg
     };
+
 }
 
 sub _CanonicalizeCommand {
@@ -653,45 +561,24 @@ sub _CanonicalizeCommand {
     return $key;
 }
 
-sub _CheckCommand {
-    my ($cmd, $val) = (lc shift, shift);
-    return 1 if $cmd =~ /^(add|del|)customfield{.*}$/i;
-    if ( grep $cmd eq lc $_, @REGULAR_ATTRIBUTES, @TIME_ATTRIBUTES, @DATE_ATTRIBUTES ) {
-        return 1 unless ref $val;
-        return (0, "Command '$cmd' doesn't support multiple values");
-    }
-    return 1 if grep $cmd eq lc $_, @LINK_ATTRIBUTES, @WATCHER_ATTRIBUTES;
-    if ( $cmd =~ /^(?:add|del)(.*)$/i ) {
-        my $cmd = $1;
-        if ( grep $cmd eq lc $_, @REGULAR_ATTRIBUTES, @TIME_ATTRIBUTES, @DATE_ATTRIBUTES ) {
-            return (0, "Command '$cmd' doesn't support multiple values");
-        }
-        return 1 if grep $cmd eq lc $_, @LINK_ATTRIBUTES, @WATCHER_ATTRIBUTES;
-    }
-
-    return (0, "Command '$cmd' is unknown");
-}
-
 sub _ReportResults {
     my %args = ( Ticket => undef, Message => undef, Results => {}, @_ );
 
     my $msg = '';
     unless ( $args{'Ticket'} ) {
-        $msg .= $args{'Results'}{'Create'}{'message'} || '';
-        $msg .= "\n" if $msg;
-        delete $args{'Results'}{'Create'};
-    }
-
-    foreach my $key ( keys %{ $args{'Results'} } ) {
-        my @records = ref $args{'Results'}->{ $key } eq 'ARRAY'?
-                         @{$args{'Results'}->{ $key }}: $args{'Results'}->{ $key };
-        foreach my $rec ( @records ) {
-            next if $rec->{'result'};
-            $msg .= "Failed command '". $key .": ". $rec->{'value'} ."'\n";
-            $msg .= "Error message: ". ($rec->{'message'}||"(no message)") ."\n\n";
+        $msg .= $args{'Results'}->{'Create'}->{message};
+    } else {
+        foreach my $key ( keys %{ $args{'Results'} } ) {
+            my @records = ref $args{'Results'}->{ $key } eq 'ARRAY'?
+                             @{$args{'Results'}->{ $key }}: $args{'Results'}->{ $key };
+            foreach my $rec ( @records ) {
+                next if $rec->{'result'};
+                $msg .= "Failed command ". $key .":". $rec->{'value'} ."\n";
+                $msg .= "Error message ". $rec->{'message'} ."\n\n";
+            }
         }
     }
-    return unless $msg && $msg !~ /^\s*$/;
+    return unless $msg;
 
     $RT::Logger->warning( $msg );
     my $ErrorsTo = RT::Interface::Email::ParseErrorsToAddressFromHead( $args{'Message'}->head );
@@ -700,7 +587,6 @@ sub _ReportResults {
         Subject     => "Extended mailgate error",
         Explanation => $msg,
         MIMEObj     => $args{'Message'},
-        Attach      => $args{'Message'}->as_string,
     );
     return;
 }
